// file SharedMemoryGuard.h
// author Edda Nourbakhsh


#pragma once


#define SharedMemorySize 65536u
/// By including and using SharedMemoryGuard all classes can ensure to use 
/// the same shared memory name by the definition "SharedMemoryName".
#define SharedMemoryName "MySharedMemory"
#define SegmentOwnerAppendix "Owner"


#include <boost/interprocess/containers/string.hpp>
#include <boost/interprocess/managed_shared_memory.hpp>


  class SharedMemoryGuard
    /// manages boost::interprocess::managed_shared_memory.
  {


  public:
    SharedMemoryGuard(char const * memoryName)
      :
      mMemoryName(memoryName),
      mHasSharedMemory(false)
      /// Default constructor
      /// constructs boost::interprocess::managed_shared_memory if necessary
      /// and opens it.
      ///
      /// \param [in] memoryName Name of 
      ///                        boost::interprocess::managed_shared_memory
    {
      try
      {
        //Open the shared memory with given name
        mMemory = boost::interprocess::managed_shared_memory(
          boost::interprocess::open_only,
          mMemoryName.c_str()
          );
        
        mHasSharedMemory = true;
      }
      catch (boost::interprocess::interprocess_exception ipe)
      {
        boost::interprocess::error_code_t ec = ipe.get_error_code();
        boost::interprocess::native_error_t nec = ipe.get_native_error();
        
        try
        {
          //Create new shared memory with given name and size
          mMemory = boost::interprocess::managed_shared_memory(
            boost::interprocess::open_or_create,
            mMemoryName.c_str(),
            SharedMemorySize
            );

          mHasSharedMemory = true;
        }
        catch (boost::interprocess::interprocess_exception ipe)
        {
          boost::interprocess::error_code_t ec = ipe.get_error_code();
          boost::interprocess::native_error_t nec = ipe.get_native_error();
        }
        catch (...)
        {
        }
      }
      catch (...)
      {
      }
    }

    ///////////////

    ~SharedMemoryGuard()
      /// Default destructor 
      /// removes boost::interprocess::managed_shared_memory if all its named 
      /// objects have been removed.
    {
      if (mHasSharedMemory)
      {
        if (mMemory.get_num_named_objects() <= 0)
        {
          bool removed = boost::interprocess::shared_memory_object::remove(mMemoryName.c_str());
          
          mHasSharedMemory = false;
        }

      }
    }

    ///////////////

  public:
    //Mutex to protect access to shared memory owner and user.
    //boost::interprocess::named_mutex mMutex;

    boost::interprocess::managed_shared_memory mMemory;
    bool mHasSharedMemory;



  private:
    boost::interprocess::string mMemoryName;


  };
